


#include <stm32f10x_lib.h>

#include "FonctionsI2C.h"
#include "Parametres.h"



/* Initialisation du module  I2C 1  du STM32F100RB en tant que  Matre 

Paramtre d'entre :

Aucun.

Valeur de retour :

Aucune.

*/
void InitialisationI2C1(void)
{
    /* I2C1 clock enable */
	RCC->APB1ENR |= (1 << 21);

    /* Enable I2C1 reset state */
	RCC->APB1RSTR |= (1 << 21);

    /* Release I2C1 from reset state */
	RCC->APB1RSTR &= ~(1 << 21);

	I2C1->CR1 = 0x0401;
	I2C1->CR2 = 0x0108;
	
	I2C1->SR1 = 0x0000;
	I2C1->SR2 = 0x0000;
	
	I2C1->CCR = 0x8006;
	
	I2C1->OAR1 = 0x4028;
	I2C1->OAR2 = 0x0000;
	
	I2C1->TRISE = 0x0003;
}

/* Lecture d'un ou de plusieurs octets grce au module  I2C 1  du STM32F100RB

Paramtres d'entre :

Adresse - Adresse de lecture, sur le bus  I2C 1  du composant que l'on souhaite lire,
Nombre - Nombre d'octets  lire,
* Receptacle - Pointeur sur une zone de la mmoire qui est destine  contenir le ou les octets qui auront t lus.

Valeur de retour :

1 si la lecture s'est effectue correctement, -1 s'il y a eu une erreur lors de la lecture.

*/
int LirePlusieursOctets(unsigned char Adresse, unsigned char Nombre, unsigned char * Receptacle)
{
    int Tampon = 0;
    int Timeout = 0;

	if(Nombre <= 2)
	{
       /* Set POS bit */
        I2C1->CR1 |= 0x0800;
		/* Rarmement de la variable qui indique le dpassement du nombre d'essais maximal */
        Timeout = 0xFFFF;
        /* Send START condition */
        I2C1->CR1 |= 0x0100;
        /* Wait until SB flag is set: EV5 */
        while((I2C1->SR1&0x0001) != 0x0001)
        {
            if(Timeout-- == 0)
			{
                return (-1);
			}
        }
		/* Rarmement de la variable qui indique le dpassement du nombre d'essais maximal */
        Timeout = 0xFFFF;
        /* Send the slave address */
        I2C1->DR = Adresse;
        /* Wait until ADDR is set: EV6 */
        while((I2C1->SR1&0x0002) != 0x0002)
        {
            if(Timeout-- == 0)
			{
                return (-1);
			}
        }
        /* Clear ADDR by reading SR2 register  */
        Tampon = I2C1->SR2;
        /* Clear ACK */
        I2C1->CR1 &= 0xFBFF;
        /* Wait until BTF is set */
        	while((I2C1->SR1 & 0x00004) != 0x000004);
        /* Program the STOP */
        I2C1->CR1 |= 0x0200;
        /* Read first data */
        Receptacle[0] = I2C1->DR;
		if(Nombre == 2)
		{
            /* Read second data */
            Receptacle[1] = I2C1->DR;
		}
        /* Make sure that the STOP bit is cleared by Hardware before CR1 write access */
        while ((I2C1->CR1&0x200) == 0x200);
        /* Enable Acknowledgement to be ready for another reception */
        I2C1->CR1  |= 0x0400;
        /* Clear POS bit */
        I2C1->CR1  &= 0xF7FF;
	}
	return 1;
}

/* Lecture d'un octet grce au module  I2C 1  du STM32F100RB

Paramtres d'entre :

Adresse - Adresse de lecture, sur le bus  I2C 1 , du composant que l'on souhaite lire.

Valeur de retour :

La donne qui a t lue si la lecture s'est effectue correctement, -1 s'il y a eu une erreur lors de la lecture.

*/
int Lire(unsigned char Adresse)
{
 	unsigned char Tampon[1];

	if(LirePlusieursOctets(Adresse, 1, Tampon) == (-1))
	{
		return (-1);
	}

	return Tampon[0];
}

/* Ecriture d'un ou de plusieurs octets grce au module  I2C 1  du STM32F100RB

Paramtres d'entre :

Adresse - Adresse d'criture, sur le bus  I2C 1 , du composant que l'on souhaite crire,
Nombre - Nombre d'octets  crire,
* Donnees - Pointeur sur une zone de la mmoire qui contient le ou les octets  crire.

Valeur de retour :

1 si l'criture s'est effectue correctement, -1 s'il y a eu une erreur lors de l'criture.

*/
int EcrirePlusieursOctets(unsigned char Adresse, unsigned char Nombre, unsigned char * Donnees) 
{
	int Tampon = 0;
    int Timeout = 0;

	int p = 0;

	/* Rarmement de la variable qui indique le dpassement du nombre d'essais maximal */
    Timeout = 0xFFFF;

    /* Send START condition */
    I2C1->CR1 |= 0x0100;
    /* Wait until SB flag is set: EV5 */
    while ((I2C1->SR1&0x0001) != 0x0001)
    {
        if (Timeout-- == 0)
		{
			InitialisationI2C1();

            return (-1);
		}
    }
    /* Send the slave address */
    I2C1->DR = Adresse;
	/* Rarmement de la variable qui indique le dpassement du nombre d'essais maximal */
    Timeout = 0xFFFF;
    /* Wait until ADDR is set: EV6 */
    while ((I2C1->SR1 &0x0002) != 0x0002)
    {
        if (Timeout-- == 0)
		{
			InitialisationI2C1();

            return (-1);
		}
    }
    /* Clear ADDR flag by reading SR2 register */
    Tampon = I2C1->SR2;
    /* Write the first data in DR register (EV8_1) */
    I2C1->DR = Donnees[0];
    /* Increment */
    p++;
    /* Decrement the number of bytes to be written */
    Nombre--;
    /* While there is data to be written */
    while(Nombre--)
    {
        /* Poll on BTF to receive data because in polling mode we can not guarantee the
          EV8 software sequence is managed before the current byte transfer completes */
        while((I2C1->SR1 & 0x00004) != 0x000004);
        /* Send the current byte */
        I2C1->DR = Donnees[p];
        /* Point to the next byte to be written */
        p++;
    }
    /* EV8_2: Wait until BTF is set before programming the STOP */
    while ((I2C1->SR1 & 0x00004) != 0x000004);
    /* Send STOP condition */
    I2C1->CR1 |= 0x0200;
    /* Make sure that the STOP bit is cleared by Hardware */
    while ((I2C1->CR1&0x200) == 0x200);

	return 1;
}

/* Ecriture d'un octet grce au module  I2C 1  du STM32F100RB

Paramtres d'entre :

Adresse - Adresse d'criture, sur le bus  I2C 1 , du composant que l'on souhaite crire,
Donnees - Donne  crire.

Valeur de retour :

1 si l'criture s'est effectue correctement, -1 s'il y a eu une erreur lors de l'criture.

*/
int Ecrire(unsigned char Adresse, unsigned char Donnee)
{
 	unsigned char Tampon[1];

	Tampon[0] = Donnee;

	return(EcrirePlusieursOctets(Adresse, 1, Tampon));
}


